#[repr(C)]pub struct BucketOptions {
pub cache_capacity: usize,
pub cache_evict_pct: usize,
pub pool_capacity: usize,
pub checkpoint_size: usize,
pub inline_size: usize,
pub split_elems: u16,
pub consolidate_threshold: u16,
pub enable_backpressure: bool,
pub _padding: [u8; 3],
}Fields
cache_capacity: usizePer-bucket target resident bytes for mapped B+Tree pages.
cache_evict_pct: usizePercentage of items evicted per round. Range is [10, 80], default is 20%
pool_capacity: usizePer-bucket pool target bytes. Default is Self::POOL_CAP
checkpoint_size: usizeMaximum bytes a single checkpoint round should emit. Default is Self::CHECKPOINT_SIZE
inline_size: usizeFor branch nodes, keys and indexes are always inlined. For leaf nodes, values smaller than
Self::MIN_INLINE_SIZE are always inlined, and values larger than Self::MAX_INLINE_SIZE
are stored as blobs. Default is Self::MIN_INLINE_SIZE
split_elems: u16Maximum number of elements in an SST (B+Tree node). Default is Self::MAX_SPLIT_ELEMS
consolidate_threshold: u16Threshold for consolidating delta chains. Range is [16, Self::split_elems / 2]
enable_backpressure: boolEnable foreground write backpressure. Default is true
_padding: [u8; 3]Implementations
Sourceimpl BucketOptions
impl BucketOptions
pub const MIN_SPLIT_ELEMS: u16 = 64
pub const MAX_SPLIT_ELEMS: u16 = 512
pub const CACHE_CAP: usize
pub const POOL_CAP: usize
pub const CHECKPOINT_SIZE: usize
pub const MIN_INLINE_SIZE: usize = 4096
pub const MAX_INLINE_SIZE: usize = 16384
pub fn new() -> Self
pub fn validate(self) -> BucketOptions