TxnView

Struct TxnView 

pub struct TxnView<'a> { /* private fields */ }
A read-only transaction (consistent view).

Implementations

Source

impl<'a> TxnView<'a>

pub fn get<K: AsRef<[u8]>>(&self, k: K) -> Result<ValRef, OpCode>

Gets the value associated with a key in this view.

Examples found in repository?
examples/demo.rs (line 36)
3fn main() -> Result<(), OpCode> {
4    let path = std::env::temp_dir().join("mace");
5    let _ = std::fs::remove_dir_all(&path);
6    let opt = Options::new(path).validate()?;
7    let db = Mace::new(opt)?;
8    let bucket = db.new_bucket("test", BucketOptions::default())?;
9
10    // start a read-write transaction
11    let kv = bucket.begin()?;
12    kv.put("foo", "bar")?;
13    kv.put("fool", "+1s")?;
14    kv.put("foolish", "elder")?;
15
16    // can't create two identical keys
17    let r = kv.put("foolish", "114514").err();
18    assert_eq!(r.unwrap(), OpCode::AbortTx);
19
20    // use `update` for exist key or use `upsert` when unsure
21    let r = kv.update("foolish", "114514");
22    assert!(r.is_ok());
23
24    let r = kv.get("foo")?;
25    assert_eq!(r.slice(), "bar".as_bytes());
26    kv.del("foolish")?;
27    kv.commit()?;
28
29    // rollback
30    let kv = bucket.begin()?;
31    kv.put("mo", "ha")?;
32    drop(kv);
33
34    // start a read-only transaction
35    let view = bucket.view()?;
36    let r = view.get("foo")?;
37    assert_eq!(r.slice(), "bar".as_bytes());
38    let r = view.get("mo");
39    assert_eq!(r.err().unwrap(), OpCode::NotFound);
40
41    // prefix scan
42    let r = view.get("foolish");
43    assert!(r.is_err() && r.err().unwrap() == OpCode::NotFound);
44    let iter = view.seek("foo");
45    assert_eq!(iter.count(), 2);
46
47    Ok(())
48}
pub fn seek<K>(&self, prefix: K) -> Iter<'_>
where K: AsRef<[u8]>,

Seeks an iterator to a key prefix in this view. prefix can’t be empty and the Iter::Item is only valid in current iteration.

NOTE: Iter will save a clone of the resource, so do not save Iter to avoid resource shortage.

Examples found in repository?
examples/demo.rs (line 44)
3fn main() -> Result<(), OpCode> {
4    let path = std::env::temp_dir().join("mace");
5    let _ = std::fs::remove_dir_all(&path);
6    let opt = Options::new(path).validate()?;
7    let db = Mace::new(opt)?;
8    let bucket = db.new_bucket("test", BucketOptions::default())?;
9
10    // start a read-write transaction
11    let kv = bucket.begin()?;
12    kv.put("foo", "bar")?;
13    kv.put("fool", "+1s")?;
14    kv.put("foolish", "elder")?;
15
16    // can't create two identical keys
17    let r = kv.put("foolish", "114514").err();
18    assert_eq!(r.unwrap(), OpCode::AbortTx);
19
20    // use `update` for exist key or use `upsert` when unsure
21    let r = kv.update("foolish", "114514");
22    assert!(r.is_ok());
23
24    let r = kv.get("foo")?;
25    assert_eq!(r.slice(), "bar".as_bytes());
26    kv.del("foolish")?;
27    kv.commit()?;
28
29    // rollback
30    let kv = bucket.begin()?;
31    kv.put("mo", "ha")?;
32    drop(kv);
33
34    // start a read-only transaction
35    let view = bucket.view()?;
36    let r = view.get("foo")?;
37    assert_eq!(r.slice(), "bar".as_bytes());
38    let r = view.get("mo");
39    assert_eq!(r.err().unwrap(), OpCode::NotFound);
40
41    // prefix scan
42    let r = view.get("foolish");
43    assert!(r.is_err() && r.err().unwrap() == OpCode::NotFound);
44    let iter = view.seek("foo");
45    assert_eq!(iter.count(), 2);
46
47    Ok(())
48}
pub fn range<K, R>(&self, range: R) -> Iter<'_>
where K: AsRef<[u8]>, R: RangeBounds<K>,